******************************************************************************
*
*
*
*
*
******************************************************************************

        implicit none

        include 'chkacc.inc'

        record /item_list/ items(1:40)

        integer*4 stat, protect, mode, flags
        integer ididx, dummy, len, usernamelen, objnamelen, type
        integer*4 privs(1:2)
        logical*1 read, write, execute, log_reason

        character*128 buffer
        character*1024 username, objname

        integer cvt_access, cvt_mode, add_id, cvt_type
        external cvt_access, cvt_mode, add_id, cvt_type
                     
*       Initialize everything

        privs(1) = 0
        privs(2) = 0
        flags = 0
        protect = -1
        mode = 0
        ididx = 0
        usernamelen = -1
        objnamelen = -1
        username = ' '                         

*       Look for the options specified

        stat = cli$present ('ACCESS')
        if (stat .eq. %loc(CLI$_PRESENT)) then
            buffer = ' '
            stat = cli$get_value ('ACCESS', %descr(buffer), len)
            if (stat .ne. SS$_NORMAL) then
                call sys$exit (%val(stat))
            endif

            protect = cvt_access (buffer(1:len))
        endif

*       Get the object type

        stat = cli$present ('TYPE')
        if (stat .eq. %loc(CLI$_PRESENT)) then
            buffer = ' '
            stat = cli$get_value ('TYPE', %descr(buffer), len)
            if (stat .ne. SS$_NORMAL) then
                call sys$exit (%val(stat))
            endif
                      
            type = cvt_type (buffer(1:len))
        endif

*       Get the object name

        stat = cli$present ('OBJECT')
        if (stat .eq. %loc(CLI$_PRESENT)) then
            buffer = ' '
            stat = cli$get_value ('OBJECT', %descr(buffer), len)
            if (stat .ne. SS$_NORMAL) then
                call sys$exit (%val(stat))
            endif

            objname = buffer(1:len)
            objnamelen = len
        endif

*       Get the username desiring access.

        stat = cli$present ('USERNAME')
        if (stat .eq. %loc(CLI$_PRESENT)) then
            buffer = ' '
            stat = cli$get_value ('USERNAME', %descr(buffer), len)
            if (stat .ne. SS$_NORMAL) then
                call sys$exit (%val(stat))
            endif

            username = buffer(1:len)
            usernamelen = len
        endif

*       Get the access mode access is desired at.

        stat = cli$present ('MODE')
        if (stat .eq. %loc(CLI$_PRESENT)) then
            buffer = ' '
            stat = cli$get_value ('MODE', %descr(buffer), len)
            if (stat .ne. SS$_NORMAL) then
                call sys$exit (%val(stat))
            endif

            mode = cvt_mode (buffer (1:len))
        endif

*       Check for guaranteed read access

        stat = cli$present ('READ')
        if (stat .eq. %loc(CLI$_PRESENT)) then
            flags = flags + CHP$M_READ
        endif

*       Check for guaranteed write access

        stat = cli$present ('WRITE')
        if (stat .eq. %loc(CLI$_PRESENT)) then
            flags = flags + CHP$M_WRITE
        endif

*       Check for guaranteed readall privs
                                
        stat = cli$present ('USEREADALL')
        if (stat .eq. %loc(CLI$_PRESENT)) then
            flags = flags + CHP$M_USEREADALL
        endif

*       Fill in the item list

        items(1).buflen = 4
        items(1).code = CHP$_ACCESS
        items(1).bufadr = %loc (protect)
        items(1).retlenadr = %loc (dummy)

        items(2).buflen = 1
        items(2).code = CHP$_ACMODE
        items(2).bufadr = %loc (mode)
        items(2).retlenadr = %loc (dummy)

        items(3).buflen = 4
        items(3).code = CHP$_FLAGS
        items(3).bufadr = %loc (flags)
        items(3).retlenadr = %loc (dummy)

        call sys$exit (%val ( sys$check_access (type, objname(1:objnamelen),
     .                                    username(1:usernamelen), items)))
        stop
        end
                     
******************************************************************************
*
*
*
*
******************************************************************************

        integer function cvt_type (buffer)
            implicit none
            include 'chkacc.inc'
            include '($acldef)'

            character*(*) buffer

            logical strcmp 
            external strcmp

            if (strcmp(buffer, 'DEVICE')) then
                cvt_type = ACL$C_DEVICE
            else if (strcmp(buffer, 'FILE')) then
                cvt_type = ACL$C_FILE
            else if (strcmp(buffer, 'GROUP_GLOBAL_SECTION')) then
                cvt_type = ACL$C_GROUP_GLOBAL_SECTION
            else if (strcmp(buffer, 'LOGICAL_NAME_TABLE')) then
                cvt_type = ACL$C_LOGICAL_NAME_TABLE
            else if (strcmp(buffer, 'SYSTEM_GLOBAL_SECTION')) then
                cvt_type = ACL$C_SYSTEM_GLOBAL_SECTION
            else
                call sys$exit (%val (%loc (CHKACC__BADTYPE)))
            endif
        end

******************************************************************************
*
*
*
*
******************************************************************************

        integer function cvt_mode (buffer)
            implicit none
            include 'chkacc.inc'
            intrinsic len

            character*(*) buffer
            integer leng
            character*10 user,super,exec,kernel
            logical strcmp
            external strcmp

            user = 'USER'
            super = 'SUPER'
            exec = 'EXEC'
            kernel = 'KERNEL'

            leng = len(buffer)
            if (strcmp (user(1:leng), buffer)) then
                cvt_mode = PSL$C_USER
            else if (strcmp (super(1:leng), buffer)) then
                cvt_mode = PSL$C_SUPER
            else if (strcmp (exec(1:leng), buffer)) then
                cvt_mode = PSL$C_EXEC
            else if (strcmp (kernel(1:leng), buffer)) then
                cvt_mode = PSL$C_KERNEL
            else
                call sys$exit (%val (%loc (CHKACC__BADMODE)))
            endif        
        end
                     
******************************************************************************
*
*
*
*                         
******************************************************************************
                          
        integer function cvt_access (buffer)
            implicit none          
            include 'chkacc.inc'

            character*(*) buffer
            character*50 name

            integer bidx, prot
            logical*1 done                                        
                                  
            logical strcmp, parse_prots
            external strcmp, parse_prots
            intrinsic len                                    

            bidx = 1

            done = parse_prots (buffer, bidx, prot)

            if (.not. done) call sys$exit (%val (%loc (CHKACC__BADACC)))

            cvt_access = ((.not. prot) .and. 31)
        end

******************************************************************************
*
*       This function parses the :RWED portion of a access specification
*       and returns, in prot, the ones complement representation of the
*       accesses specified.
*
******************************************************************************

        logical function parse_prots (buffer, bidx, prot)
            implicit none
			include '($armdef)'
            character*(*) buffer
            intrinsic len
            integer bidx, prot
            logical*1 done
            character ch

            prot = 31
            do while (.not. done)
                ch = buffer(bidx:bidx)
                done = (bidx .gt. len(buffer))
                if (.not. done) then
                    if (ch .eq. 'C') then
                        prot = (prot .and. (.not. ARM$M_CONTROL))
                    else if (ch .eq. 'R') then
                        prot = (prot .and. (.not. ARM$M_READ))
                    else if (ch .eq. 'W') then
                        prot = (prot .and. (.not. ARM$M_WRITE))
                    else if (ch .eq. 'E') then
                        prot = (prot .and. (.not. ARM$M_EXECUTE))
                    else if (ch .eq. 'D') then
                        prot = (prot .and. (.not. ARM$M_DELETE))
                    else
                        done = .true.
                    endif
                    if (.not. done) bidx = bidx + 1       
                endif                       
            end do
            prot = (prot .and. 31)
            parse_prots = (bidx .gt. len(buffer))
        end             


******************************************************************************
*
*       Function:   STRCMP
*       Usage:      if (strcmp (character*(*), character*(*))) then
*       Purpose:    This routine performs a case blind comparison of the
*                   two strings passed, and returns a boolean value indicating
*                   whether or not they are equal (ignoring case).
*       Output:     Logical, true or false.
*
******************************************************************************

        logical function strcmp (str1, str2)
            implicit none
        
            character*(*) str1, str2
            external str$case_blind_compare
            integer str$case_blind_compare, stat

            stat = str$case_blind_compare (str1, str2)

            strcmp = .true.
            if (stat.ne.0) strcmp = .false.
        end
