******************************************************************************
*
*       Function:   GET_INFO
*       Usage:      call get_info (stream, data_descr, request, idx)
*       Purpose:    This routine is called by routines which need to get
*                   the values that the JBC sends us with each request.
*                   'stream' is the stream identifier, 'data' is a descriptor
*                   for a buffer to contain the JBC information, request is
*                   the request that the JBC sent to us, and idx is either
*                   the index of a process structure that will be filled with
*                   task related information, or -1, if the task related
*                   information is not desired.
*       Output:     The streams(stream) structure member, and possibly the
*                   procs(idx) structure member contain the the information
*                   that the JBC sent to us with the request.
*
******************************************************************************

        subroutine get_info (stream, data, request, idx)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            integer stream, request, size, context, stat, idx, item
            logical done
            integer*2 wsize
            character*512 buffer
            record /dynam/ data
            external copy_quoted

            done = .false.
            context = 0

*           Loop until all information is processed.

            do while (.not. done)

*               Copy an item from data into buffer.

                stat = smb$read_message_item (data, context, item,
     .                                     %descr (buffer), wsize)

*               Convert the integer*2 size to integer*4

                size = wsize

*               Check to see if there are any more items.

                done = (stat .eq. %loc (SMB$_NOMOREITEMS))

*               If there were, then process the current one.

                if (.not. done) then
                    if (item .eq. SMBMSG$K_FILE_SPECIFICATION) then
                        call memcpy (%descr(streams(stream).fspec),
     .                          %ref(buffer), size, %val (32))
                    else if (item .eq. SMBMSG$K_QUEUE) then
                        call memcpy (%descr(streams(stream).queue),
     .                          %ref(buffer), size, %val (32))
                    else if (item .eq. SMBMSG$K_EXECUTOR_QUEUE) then
                        call memcpy (%descr(streams(stream).exec_queue),
     .                          %ref(buffer), size, %val (32))

*                   Get the process stuff only if it is asked for.

                    else if (idx .ne. -1) then

                        if (item .eq. SMBMSG$K_ACCOUNT_NAME) then
                            call memcpy (
     .                              %descr(procs(idx).accnam), 
     .                              %ref(buffer), size, %val (32))
                        else if (item .eq. SMBMSG$K_USER_NAME) then
                            call memcpy (
     .                              %descr(procs(idx).username),
     .                              %ref(buffer), size, %val (32))
                            call memcpy (
     .                              %descr(streams(stream).sub_user),
     .                              %ref(buffer), size, %val (32))
                        else if (item .eq. SMBMSG$K_UIC) then
                            call memcpy (%descr(procs(idx).sub_uic),
     .                          %ref(buffer), size)
                            call memcpy (%descr(streams(stream).sub_uic),
     .                          %ref(buffer), size)
                        else if (item .eq. SMBMSG$K_PRIORITY) then  
                            call memcpy (
     .                          %descr(procs(idx).prio),
     .                          %ref(buffer), size)
                        else if (item .eq. SMBMSG$K_PARAMETER_1) then
                            call copy_quoted (procs(idx).param(1), buffer, size)
                            procs(idx).paramlen(1) = size
                        else if (item .eq. SMBMSG$K_PARAMETER_2) then
                            call copy_quoted (procs(idx).param(2), buffer, size)
                            procs(idx).paramlen(2) = size
                        else if (item .eq. SMBMSG$K_PARAMETER_3) then
                            call copy_quoted (procs(idx).param(3), buffer, size)
                            procs(idx).paramlen(3) = size
                        else if (item .eq. SMBMSG$K_PARAMETER_4) then
                            call copy_quoted (procs(idx).param(4), buffer, size)
                            procs(idx).paramlen(4) = size
                        else if (item .eq. SMBMSG$K_PARAMETER_5) then
                            call copy_quoted (procs(idx).param(5), buffer, size)
                            procs(idx).paramlen(5) = size
                        else if (item .eq. SMBMSG$K_PARAMETER_6) then
                            call copy_quoted (procs(idx).param(6), buffer, size)
                            procs(idx).paramlen(6) = size
                        else if (item .eq. SMBMSG$K_PARAMETER_7) then
                            call copy_quoted (procs(idx).param(7), buffer, size)
                            procs(idx).paramlen(7) = size
                        else if (item .eq. SMBMSG$K_PARAMETER_8) then
                            call copy_quoted (procs(idx).param(8), buffer, size)
                            procs(idx).paramlen(8) = size
                        else if (item .eq. SMBMSG$K_JOB_NAME) then
                            call memcpy (
     .                          %descr(procs(idx).job_name),
     .                          %ref(buffer), size, %val (32))
                        else if (item .eq. SMBMSG$K_ENTRY_NUMBER) then
                            call memcpy (
     .                          %descr(procs(idx).entry),
     .                          %ref(buffer), size)
                        else if (item .eq. SMBMSG$K_AFTER_TIME) then
                            call memcpy (
     .                          %descr(procs(idx).delta_t),
     .                          %ref(buffer), size)
                        else
                            size = 0
                        endif
                    endif
                endif
            end do
        end

******************************************************************************
*
*       Function:   COPY_QUOTED
*       Usage:      call copy_quoted (dest_descr, buf_descr, size)
*       Purpose:    This routine copies a string of character 'size' long
*                   from 'buf' to 'str'.  Any '"' character found is doubled
*                   in str so that the string can be passed to DCL, and the
*                   quotes will be maintained.
*       Output:     'str' contains the string, and 'size' contains the new
*                   length of 'str'.
*
******************************************************************************

        subroutine copy_quoted (str, buf, size)
            implicit none
            character*(*) str, buf
            integer size, i, j

            str = ' '

            i=1
            j=1
            do while (j .le. size)
                str(i:i) = buf(j:j)
                if (str(i:i) .eq. '"') then
                    i = i + 1
                    str(i:i) = '"'
                endif
                i = i + 1
                j = j + 1
            end do
            size = i - 1
        end
