******************************************************************************
*
*       Function:   PROC_DONE
*       Usage:      Called as an AST service routine.
*       Purpose:    This routine is used as the AST routine of a READATTN
*                   AST request on the SYS$INPUT mailbox of a subprocess.
*                   After the subprocess has been given it's work to do, this
*                   routine is set up to be the point of notification that
*                   the subprocess has completed it's task, i.e. it issues
*                   another read on the mailbox.
*       Output:     The status of the process is set to available, and if
*                   so desired, it's process name is changed to reflect that
*                   it is IDLE.  Finally, the JBC is told that the task that
*                   was associated with this subprocess has completed.
*
******************************************************************************

        subroutine proc_done (proc)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            record /sub_proc/ proc
            integer daytim(1:2), clen, stream, status
            character*(BUFSIZ) cmd
            external sys$setimr, signal, expire_proc, prcnam, task_complete
            external log_sys
            character*2 prcnam
            integer sys$setimr

            status = SS$_NORMAL
            if ((proc.iosb.cond .and. 1).ne.1) then
                call log_sys (%val (proc.iosb.cond),
     .                          'Error in $QIO waiting for task to finish')
                status = proc.iosb.cond
            endif

*           Mark the process as available

            proc.ready = .false.
            proc.job_name = 'NO_JOB'
            proc.entry = 0
            proc.starting = .false.
            proc.avail = .true.

            stream = proc.stream
            if (.not. proc.dedicated) then
                proc.stream = MAX_STREAMS+1
            endif

            cmd='SET PROC/NAME="IDLE_'//prcnam(proc.idx)//'"'
            clen = 23
            call signal (sys$qiow (
     .                      %val(proc.efn),
     .                      %val(proc.mailchan),
     .                      %val(IO$_WRITELBLK+IO$M_NOW),
     .                      proc.iosb,
     .                      , ,
     .                      %ref(cmd),
     .                      %val(clen),
     .                      , , , ))

*           Get 10 minutes of delta time in a quadword

            call delta_secs (daytim, streams(stream).expire)

*           Set a new expire timeout.

            call signal (sys$setimr (, %ref(daytim), expire_proc, proc))

*           Tell the JBC that we are done

            if (streams(stream).pause_time .eq. 0) then
                call reply (status, SMBMSG$K_TASK_COMPLETE, stream)
            else
                call delta_secs (daytim, streams(stream).pause_time)

*               Set a new expire timeout.

                call signal (sys$setimr (, %ref(daytim), task_complete, stream))
            endif
        end

*
*       Tell the JBC that the task for the indicate stream is complete
*

        subroutine task_complete (stream)
            implicit none
            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            integer stream

            call reply (SS$_NORMAL, SMBMSG$K_TASK_COMPLETE, stream)
        end
