******************************************************************************
*
*       Function:   START_TASK
*       Usage:      call start_task (SMB_stream_id, data_descr, SMB_request)
*       Purpose:    This routine is used to process the JBC's START_TASK
*                   requests.  It makes use of many lower leve routines to
*                   obtain a subprocess to process the job in, and then
*                   ASK's for a READATTN AST on the processes SYS$INPUT
*                   mailbox so that we can send the right stuff to the
*                   process when it becomes ready.
*       Output:     A pending AST to be delivered to the function PROC_READY.
*
******************************************************************************

        subroutine start_task (stream, data, request)
            implicit none

            include 'defs.inc'
            include 'struct.inc'
            include 'extern.inc'
            include 'global.inc'

            integer stream, request, idx, i, last_stat
            record /dynam/ data

            external get_info, get_proc, start_proc, reply, signal
            external proc_ready, log_sys, getuinfo
            integer get_proc
            logical start_proc

*           Get initial info on job to know how to choose a process.  Make
*           Use of a dummy process slot to cause some other information
*           to be written into the streams structure.

            call get_info (stream, data, request, MAXPROCS+1)
            if (streams(stream).use_sub_user) then
                streams(stream).user = streams(stream).sub_user
                call getuinfo (streams(stream))
            endif

*           Get the index of a process slot that we can use.  There is always
*           the possibility that the process will be deleted before we get to
*           start_proc(), but that is life.  The code should NOT depend on
*           there being a process running in a slot until after the return
*           from start_proc.

            idx = get_proc (stream, last_stat)

*           If none left, or problems creating one, abort.

            if (idx .eq. -1) then
                call reply (last_stat, request, stream)
                call log_sys (%val(last_stat), 'Can''t find a new process slot')
                return
            endif

            last_idx = idx

*           Zap the parameters to zero length.

            do i=1,MAXPARAMS
                procs(idx).paramlen(i) = 0
            end do

*           Place the necessary information into the procs structure.
*           The information includes the parameters, the UIC, the USERNAME,
*           and other assorted information that the JBC sends to us.

            call get_info (stream, data, request, idx)

*           Start the process.  The starting flag is used to indicate to
*           that a process is in its beginning stages, and could possibly
*           exit after being started successfully.

            if (.not. start_proc (procs(idx), stream, last_stat)) then
                call reply (last_stat, request, stream)
                call log_sys (%val(last_stat), 'Can''t start a new process')
                procs(idx).avail = .true.
                procs(idx).started = .false.
                return
            endif

*           Set the READATTN ast on the input mailbox.

            call get_read_attn (procs(idx), proc_ready)

*           Note that at the completion of the READATTN AST, the JBC will
*           then be told SMBMSG$K_START_TASK.  Only then will it know that the
*           task is being processed.  We will then know that the subprocess
*           is running so that we can write the necessary stuff down the
*           mailbox (see PROC_READY()).
        end
